@@ -19,7 +19,10 @@ verify_ssl = True
 proxies: Dict[str, str] = {}
 
 #: Version of the SPGCI Pkg
-version = "0.0.1"
+version = "0.0.9"
+
+#: time to sleep between api calls
+sleep_time = 0
 
 
 def set_credentials(un: str, pw: str, apikey: str) -> None:
