@@ -1,10 +1,12 @@
+from __future__ import annotations
 from spgci.utilities import list_to_filter
 from spgci.api_client import get_data, Paginator
-from spgci.types import ContractType, AssessmentFrequency
 from typing import List, Optional, Union
 import pandas as pd
+from pandas import Series
 from requests import Response
 from datetime import date
+from enum import Enum
 
 
 class MarketData:
@@ -13,21 +15,63 @@ class MarketData:
 
     Includes
     --------
-    Methods to get symbol data (such as description, currency, unit of measure)\n
-    Methods to get assessment data (such as the assessment of a given symbol on a particular date)
+    ``ContractType`` enum for contract types.\n
+    ``AssessmentFrequency`` enum for assessment frequencies.\n
+    ``get_assessments_by_symbol_current`` get current assessments for a list of symbols.
+    ``get_assessments_by_symbol_current`` get current assessments for a list of symbols.
+    ``get_assessments_by_mdc_current`` get current assessments for all symbols in an MDC (Market Data Category).
+    ``get_assessments_by_mdc_historical`` get historical assessments for all symbols in an MDC (Market Data Category).
 
     """
 
     _path = "market-data/v3/value/"
     _ref_path = "market-data/reference-data/v3/search"
 
+    class ContractType(Enum):
+        """Contract Type"""
+
+        Spot = "spot"
+        Forward = "forward"
+        Future = "future"
+        Swap = "swap"
+        Strip = "strip"
+        CFD = "cfd"
+        Index = "index"
+        OfficialSellingPrice = "official selling price"
+        Yield = "yield"
+        Contract = "contract"
+        ESS = "ess"
+        Prompt = "prompt"
+        Statistic = "statistic"
+        EFP = "efp"
+        Netback = "netback"
+        EFS = "efs"
+        Rack = "rack"
+
+    class AssessmentFrequency(Enum):
+        """Asessment Frequency"""
+
+        Intraday = "Intraday"
+        Daily = "Daily (7 day)"
+        DailyWeekday = "Daily (weekday)"
+        DailyBidweekOnly = "Daily (bidweek only)"
+        SemiWeekly = "Semi-weekly"
+        Weekly = "Weekly"
+        SemiMonthly = "Semi-monthly"
+        Monthly = "Monthly"
+        EveryOtherMonth = "Every other month"
+        Quarterly = "Quarterly"
+        SemiAnnual = "Semi-annual"
+        Yearly = "Yearly"
+
     @staticmethod
     def _convert_to_df(resp: Response) -> pd.DataFrame:
         j = resp.json()
         df = pd.json_normalize(j["results"], record_path=["data"], meta="symbol")  # type: ignore
 
-        df["assessDate"] = pd.to_datetime(df["assessDate"])  # type: ignore
-        df["modDate"] = pd.to_datetime(df["modDate"])  # type: ignore
+        if len(df) > 0:
+            df["assessDate"] = pd.to_datetime(df["assessDate"])  # type: ignore
+            df["modDate"] = pd.to_datetime(df["modDate"])  # type: ignore
 
         return df
 
@@ -42,9 +86,14 @@ class MarketData:
         return Paginator(True, "page", total_pages=total_pages)
 
     @staticmethod
-    def _search_to_def(resp: Response) -> pd.DataFrame:
+    def _search_to_df(resp: Response) -> pd.DataFrame:
         j = resp.json()
         df = pd.DataFrame(j["results"])
+
+        if len(df) > 0:
+            cols = ["symbol", "description"]
+            df: pd.DataFrame = df[cols + [x for x in df.columns if x not in cols]]  # type: ignore
+
         return df
 
     @staticmethod
@@ -60,13 +109,13 @@ class MarketData:
     def get_assessments_by_symbol_current(
         self,
         *,
-        symbol: Optional[Union[List[str], str]] = None,
-        bate: Optional[Union[List[str], str]] = None,
+        symbol: Optional[Union[list[str], "Series[str]", str]] = None,
+        bate: Optional[Union[list[str], "Series[str]", str]] = None,
         filter_exp: Optional[str] = None,
         page: int = 1,
         page_size: int = 10000,
         raw: bool = False,
-        paginate: bool = True,
+        paginate: bool = False,
     ) -> Union[pd.DataFrame, Response]:
         """
         Fetch Current Assessments by Symbol from the Market Data API.
@@ -76,9 +125,9 @@ class MarketData:
 
         Parameters
         ----------
-        symbol : Optional[Union[List[str], str]], optional
+        symbol : Optional[Union[list[str], Series[str], str]], optional
             filter by symbol, by default None
-        bate : Optional[Union[List[str], str]], optional
+        bate : Optional[Union[list[str], Series[str], str]], optional
             filter by bate, by default None
         filter_exp : Optional[str], optional
             pass-thru ``filter`` query param to use a handcrafted filter expression, by default None
@@ -87,7 +136,7 @@ class MarketData:
         page_size : int, optional
             pass-thru ``pageSize`` query param to request a particular page size, by default 1000
         paginate : bool, optional
-            whether to auto-paginate the response, by default True
+            whether to auto-paginate the response, by default False
         raw : bool, optional
             return a ``requests.Response`` instead of a ``DataFrame``, by default False
 
@@ -102,22 +151,22 @@ class MarketData:
         Examples
         --------
         **Simple**
-        >>> MarketData.get_assessments_symbol_current(symbol="PCAAS00")
+        >>> MarketData().get_assessments_symbol_current(symbol="PCAAS00")
 
         **Multiple Symbols and Bates**
-        >>> MarketData.get_assessments_symbol_current(symbol=["PCAAS00", "PCAAT00"], bate=["c", "h"])
+        >>> MarketData().get_assessments_symbol_current(symbol=["PCAAS00", "PCAAT00"], bate=["c", "h"])
         """
-        paramList: List[str] = []
+        filter_params: List[str] = []
 
-        if symbol:
-            paramList.append(list_to_filter("symbol", symbol))
-        if bate:
-            paramList.append(list_to_filter("bate", bate))
+        filter_params.append(list_to_filter("symbol", symbol))
+        filter_params.append(list_to_filter("bate", bate))
+
+        filter_params = [fp for fp in filter_params if fp != ""]
 
         if filter_exp is None:
-            filter_exp = " AND ".join(paramList)
+            filter_exp = " AND ".join(filter_params)
         else:
-            filter_exp += " AND " + " AND ".join(paramList)
+            filter_exp += " AND " + " AND ".join(filter_params)
 
         endpoint_path = "current/symbol"
         params = {"filter": filter_exp, "page": page, "pageSize": page_size}
@@ -134,8 +183,8 @@ class MarketData:
     def get_assessments_by_symbol_historical(
         self,
         *,
-        symbol: Optional[Union[List[str], str]] = None,
-        bate: Optional[Union[List[str], str]] = None,
+        symbol: Optional[Union[list[str], "Series[str]", str]] = None,
+        bate: Optional[Union[list[str], "Series[str]", str]] = None,
         assess_date: Optional[date] = None,
         assess_date_lt: Optional[date] = None,
         assess_date_lte: Optional[date] = None,
@@ -144,7 +193,7 @@ class MarketData:
         filter_exp: Optional[str] = None,
         page: int = 1,
         page_size: int = 10000,
-        paginate: bool = True,
+        paginate: bool = False,
         raw: bool = False,
     ) -> Union[pd.DataFrame, Response]:
         """
@@ -155,9 +204,9 @@ class MarketData:
 
         Parameters
         ----------
-        symbol : Optional[Union[List[str], str]], optional
+        symbol : Optional[Union[list[str], Series[str], str]], optional
             filter by symbol, by default None
-        bate : Optional[Union[List[str], str]], optional
+        bate : Optional[Union[list[str], Series[str], str]], optional
             filter by bate, by default None
         assess_date : Optional[date], optional
             filter by ``assessDate = x`` , by default None
@@ -176,7 +225,7 @@ class MarketData:
         page_size : int, optional
             pass-thru ``pageSize`` query param to request a particular page size, by default 1000
         paginate : bool, optional
-            whether to auto-paginate the response, by default True
+            whether to auto-paginate the response, by default False
         raw : bool, optional
             return a ``requests.Response`` instead of a ``DataFrame``, by default False
 
@@ -191,38 +240,38 @@ class MarketData:
         Examples
         --------
         **Simple**
-        >>> MarketData.get_assessments_symbol_historical(symbol="PCAAS00")
+        >>> MarketData().get_assessments_symbol_historical(symbol="PCAAS00")
 
         **Multiple Symbols and Bates**
-        >>> MarketData.get_assessments_symbol_historical(symbol=["PCAAS00", "PCAAT00"], bate=["c", "h"])
+        >>> MarketData().get_assessments_symbol_historical(symbol=["PCAAS00", "PCAAT00"], bate=["c", "h"])
 
         **Date Range**
         >>> d1 = date(2023, 1, 1)
         >>> d2 = date(2023, 2, 1)
-        >>> MarketData.get_assessments_symbol_historical(symbol=["PCAAS00", "PCAAT00"], assess_date_gte=d1, assess_date_lte=d2])
+        >>> MarketData().get_assessments_symbol_historical(symbol=["PCAAS00", "PCAAT00"], assess_date_gte=d1, assess_date_lte=d2])
         """
-        paramList: List[str] = []
+        filter_params: List[str] = []
 
-        if symbol:
-            paramList.append(list_to_filter("symbol", symbol))
-        if bate:
-            paramList.append(list_to_filter("bate", bate))
+        filter_params.append(list_to_filter("symbol", symbol))
+        filter_params.append(list_to_filter("bate", bate))
 
         if assess_date != None:
-            paramList.append(f'assessDate: "{assess_date}"')
+            filter_params.append(f'assessDate: "{assess_date}"')
         if assess_date_gt != None:
-            paramList.append(f'assessDate > "{assess_date_gt}"')
+            filter_params.append(f'assessDate > "{assess_date_gt}"')
         if assess_date_gte != None:
-            paramList.append(f'assessDate >= "{assess_date_gte}"')
+            filter_params.append(f'assessDate >= "{assess_date_gte}"')
         if assess_date_lt != None:
-            paramList.append(f'assessDate < "{assess_date_lt}"')
+            filter_params.append(f'assessDate < "{assess_date_lt}"')
         if assess_date_lte != None:
-            paramList.append(f'assessDate <= "{assess_date_lte}"')
+            filter_params.append(f'assessDate <= "{assess_date_lte}"')
+
+        filter_params = [fp for fp in filter_params if fp != ""]
 
         if filter_exp is None:
-            filter_exp = " AND ".join(paramList)
+            filter_exp = " AND ".join(filter_params)
         else:
-            filter_exp += " AND " + " AND ".join(paramList)
+            filter_exp += " AND " + " AND ".join(filter_params)
 
         endpoint_path = "history/symbol"
         params = {"filter": filter_exp, "page": page, "pageSize": page_size}
@@ -239,11 +288,11 @@ class MarketData:
         self,
         *,
         mdc: str,
-        bate: Optional[List[str]] = None,
+        bate: Optional[Union[list[str], "Series[str]", str]] = None,
         filter_exp: Optional[str] = None,
         page: int = 1,
         page_size: int = 10000,
-        paginate: bool = True,
+        paginate: bool = False,
         raw: bool = False,
     ) -> Union[pd.DataFrame, Response]:
         """
@@ -256,7 +305,7 @@ class MarketData:
         ----------
         mdc : str
             filter by Market Data Category
-        bate : Optional[List[str]], optional
+        bate : Optional[Union[list[str], Series[str], str], optional
             filter by bate, by default None
         filter_exp : Optional[str], optional
             pass-thru ``filter`` query param to use a handcrafted filter expression, by default None
@@ -265,7 +314,7 @@ class MarketData:
         page_size : int, optional
             pass-thru ``pageSize`` query param to request a particular page size, by default 1000
         paginate : bool, optional
-            whether to auto-paginate the response, by default True
+            whether to auto-paginate the response, by default False
         raw : bool, optional
             return a ``requests.Response`` instead of a ``DataFrame``, by default False
 
@@ -280,23 +329,24 @@ class MarketData:
         Examples
         --------
         **Simple**
-        >>> MarketData.get_assessments_by_mdc_current(mdc="ET")
+        >>> MarketData().get_assessments_by_mdc_current(mdc="ET")
 
         **Include bate**
-        >>> MarketData.get_assessments_by_mdc_current(mdc="ET", bate=['c', 'u'])
+        >>> MarketData().get_assessments_by_mdc_current(mdc="ET", bate=['c', 'u'])
 
         **Turn off auto pagination**
-        >>> MarketData.get_assessments_by_mdc_current(mdc="ET", paginate=false)
+        >>> MarketData().get_assessments_by_mdc_current(mdc="ET", paginate=false)
         """
-        paramList: List[str] = []
-        paramList.append(f'MDC: "{mdc}"')
-        if bate:
-            paramList.append(list_to_filter("bate", bate))
+        filter_params: List[str] = []
+        filter_params.append(f'MDC: "{mdc}"')
+        filter_params.append(list_to_filter("bate", bate))
+
+        filter_params = [fp for fp in filter_params if fp != ""]
 
         if filter_exp is None:
-            filter_exp = " AND ".join(paramList)
+            filter_exp = " AND ".join(filter_params)
         else:
-            filter_exp += " AND " + " AND ".join(paramList)
+            filter_exp += " AND " + " AND ".join(filter_params)
 
         endpoint_path = "current/mdc"
         params = {"filter": filter_exp, "page": page, "pageSize": page_size}
@@ -318,11 +368,11 @@ class MarketData:
         assess_date_lte: Optional[date] = None,
         assess_date_gt: Optional[date] = None,
         assess_date_gte: Optional[date] = None,
-        bate: Optional[List[str]] = None,
+        bate: Optional[Union[list[str], "Series[str]", str]] = None,
         filter_exp: Optional[str] = None,
         page: int = 1,
         page_size: int = 10000,
-        paginate: bool = True,
+        paginate: bool = False,
         raw: bool = False,
     ) -> Union[pd.DataFrame, Response]:
         """
@@ -345,7 +395,7 @@ class MarketData:
             filter by ``assessDate > x``, by default None
         assess_date_gte : Optional[date], optional
             filter by ``assessDate >= x``, by default None
-        bate : List[str], optional
+        bate : Union[list[str], Series[str], str], optional
             filter by bate, by default []
         filter_exp : Optional[str], optional
             pass-thru ``filter`` query param to use a handcrafted filter expression, by default None
@@ -354,7 +404,7 @@ class MarketData:
         page_size : int, optional
             pass-thru ``pageSize`` query param to request a particular page size, by default 1000
         paginate : bool, optional
-            whether to auto-paginate the response, by default True
+            whether to auto-paginate the response, by default False
         raw : bool, optional
             return a ``requests.Response`` instead of a ``DataFrame``, by default False
 
@@ -369,33 +419,34 @@ class MarketData:
         Examples
         --------
         **Simple**
-        >>> MarketData.get_assessments_by_mdc_historical(mdc="ET", assess_date=date(2023,2,1))
+        >>> MarketData().get_assessments_by_mdc_historical(mdc="ET", assess_date=date(2023,2,1))
 
         **Date Range**
         >>> d1 = date(2023,1,1)
         >>> d2 = date(2023,2,1)
-        >>> MarketData.get_assessments_by_mdc_historical(mdc="ET", assess_date_gte=d1, assess_date_lte=d2)
+        >>> MarketData().get_assessments_by_mdc_historical(mdc="ET", assess_date_gte=d1, assess_date_lte=d2)
         """
-        paramList: List[str] = []
-        paramList.append(f'MDC: "{mdc}"')
-        if bate:
-            paramList.append(list_to_filter("bate", bate))
+        filter_params: List[str] = []
+        filter_params.append(f'MDC: "{mdc}"')
+        filter_params.append(list_to_filter("bate", bate))
 
         if assess_date != None:
-            paramList.append(f'assessDate: "{assess_date}"')
+            filter_params.append(f'assessDate: "{assess_date}"')
         if assess_date_gt != None:
-            paramList.append(f'assessDate > "{assess_date_gt}"')
+            filter_params.append(f'assessDate > "{assess_date_gt}"')
         if assess_date_gte != None:
-            paramList.append(f'assessDate >= "{assess_date_gte}"')
+            filter_params.append(f'assessDate >= "{assess_date_gte}"')
         if assess_date_lt != None:
-            paramList.append(f'assessDate < "{assess_date_lt}"')
+            filter_params.append(f'assessDate < "{assess_date_lt}"')
         if assess_date_lte != None:
-            paramList.append(f'assessDate <= "{assess_date_lte}"')
+            filter_params.append(f'assessDate <= "{assess_date_lte}"')
+
+        filter_params = [fp for fp in filter_params if fp != ""]
 
         if filter_exp is None:
-            filter_exp = " AND ".join(paramList)
+            filter_exp = " AND ".join(filter_params)
         else:
-            filter_exp += " AND " + " AND ".join(paramList)
+            filter_exp += " AND " + " AND ".join(filter_params)
 
         endpoint_path = "history/mdc"
         params = {"filter": filter_exp, "page": page, "pageSize": page_size}
@@ -412,23 +463,29 @@ class MarketData:
         self,
         *,
         q: Optional[str] = None,
-        commodity: Optional[Union[List[str], str]] = None,
+        commodity: Optional[Union[list[str], "Series[str]", str]] = None,
         contract_type: Optional[
-            Union[List[str], List[ContractType], str, ContractType]
+            Union[list[str], list[ContractType], "Series[str]", str, ContractType]
         ] = None,
-        currency: Optional[Union[List[str], str]] = None,
-        uom: Optional[Union[List[str], str]] = None,
-        symbol: Optional[Union[List[str], str]] = None,
-        delivery_region_basis: Optional[Union[List[str], str]] = None,
-        curve_code: Optional[Union[List[str], str]] = None,
-        mdc: Optional[Union[List[str], str]] = None,
+        currency: Optional[Union[list[str], "Series[str]", str]] = None,
+        uom: Optional[Union[list[str], "Series[str]", str]] = None,
+        symbol: Optional[Union[list[str], "Series[str]", str]] = None,
+        delivery_region_basis: Optional[Union[list[str], "Series[str]", str]] = None,
+        curve_code: Optional[Union[list[str], "Series[str]", str]] = None,
+        mdc: Optional[Union[list[str], "Series[str]", str]] = None,
         assessment_frequency: Optional[
-            Union[List[str], List[AssessmentFrequency], str, AssessmentFrequency]
+            Union[
+                list[str],
+                list[AssessmentFrequency],
+                "Series[str]",
+                str,
+                AssessmentFrequency,
+            ]
         ] = None,
         filter_exp: Optional[str] = None,
         page: int = 1,
         page_size: int = 1000,
-        paginate: bool = True,
+        paginate: bool = False,
         raw: bool = False,
     ) -> Union[pd.DataFrame, Response]:
         """
@@ -438,23 +495,23 @@ class MarketData:
         ----------
         q : Optional[str], optional
             filter across fields using free text search, by default None
-        commodity : Optional[Union[List[str], str]], optional
+        commodity : Optional[Union[list[str], Series[str], str]], optional
             filter by commodity, by default None
-        contract_type : Optional[ Union[List[str], List[ContractType], str, ContractType] ], optional
+        contract_type : Optional[Union[list[str], list[ContractType], Series[str], str, ContractType] ], optional
             filter by contract type, by default None
-        currency : Optional[Union[List[str], str]], optional
+        currency : Optional[Union[list[str], Series[str], str]], optional
             filter by currency, by default None
-        uom : Optional[Union[List[str], str]], optional
+        uom : Optional[Union[list[str], Series[str], str]], optional
             filter by unit of measure, by default None
-        symbol : Optional[Union[List[str], str]], optional
+        symbol : Optional[Union[list[str], Series[str], str]], optional
             filter by symbol, by default None
-        delivery_region_basis : Optional[Union[List[str], str]], optional
+        delivery_region_basis : Optional[Union[list[str], Series[str], str]], optional
             filter by delivery region basis, by default None
-        curve_code : Optional[Union[List[str], str]], optional
+        curve_code : Optional[Union[list[str], Series[str], str]], optional
             filter by curve code, by default None
-        mdc : Optional[Union[List[str], str]], optional
+        mdc : Optional[Union[list[str], Series[str], str]], optional
             filter by Market Data Category, by default None
-        assessment_frequency: Optional[Union[List[str], List[AssessmentFrequency], str, AssessmentFrequency]], optional
+        assessment_frequency: Optional[Union[list[str], list[AssessmentFrequency], Series[str], str, AssessmentFrequency]], optional
             filter by Assessment Frequency, by default None
         filter_exp : Optional[str], optional
             pass-thru ``filter`` query param to use a handcrafted filter expression, by default None
@@ -463,7 +520,7 @@ class MarketData:
         page_size : int, optional
             pass-thru ``pageSize`` query param to request a particular page size, by default 1000
         paginate : bool, optional
-            whether to auto-paginate the response, by default True
+            whether to auto-paginate the response, by default False
         raw : bool, optional
             return a ``requests.Response`` instead of a ``DataFrame``, by default False
 
@@ -477,42 +534,39 @@ class MarketData:
         Examples
         --------
         **Free text search**
-        >>> MarketData.get_symbols(q="Brent")
+        >>> MarketData().get_symbols(q="Brent")
 
         **Using String**
-        >>> MarketData.get_symbols(contract_type="Forward")
+        >>> MarketData().get_symbols(contract_type="Forward")
 
         **Using List**
-        >>> MarketData.get_symbols(currency=["USD", "EUR"])
+        >>> MarketData().get_symbols(currency=["USD", "EUR"])
 
         **Using Enum**
-        >>> MarketData.get_symbols(currency=["USD", "EUR"], contract_type=[spgci.ContractType.Forward, spgci.ContractType.Spot])
+        >>> MarketData().get_symbols(currency=["USD", "EUR"], contract_type=[MarketData.ContractType.Forward, MarketData.ContractType.Spot])
         """
-        paramList: List[str] = []
+        filter_params: List[str] = []
 
-        if commodity:
-            paramList.append(list_to_filter("commodity", commodity))
-        if contract_type:
-            paramList.append(list_to_filter("contract_type", contract_type))
-        if currency:
-            paramList.append(list_to_filter("currency", currency))
-        if uom:
-            paramList.append(list_to_filter("uom", uom))
-        if delivery_region_basis:
-            paramList.append(list_to_filter("delivery_region_basis", delivery_region_basis))
-        if curve_code:
-            paramList.append(list_to_filter("curve_code", curve_code))
-        if symbol:
-            paramList.append(list_to_filter("symbol", symbol))
-        if mdc:
-            paramList.append(list_to_filter("mdc", mdc))
-        if assessment_frequency:
-            paramList.append(list_to_filter("assessment_frequency", assessment_frequency))
+        filter_params.append(list_to_filter("commodity", commodity))
+        filter_params.append(list_to_filter("contract_type", contract_type))
+        filter_params.append(list_to_filter("currency", currency))
+        filter_params.append(list_to_filter("uom", uom))
+        filter_params.append(
+            list_to_filter("delivery_region_basis", delivery_region_basis)
+        )
+        filter_params.append(list_to_filter("curve_code", curve_code))
+        filter_params.append(list_to_filter("symbol", symbol))
+        filter_params.append(list_to_filter("mdc", mdc))
+        filter_params.append(
+            list_to_filter("assessment_frequency", assessment_frequency)
+        )
+
+        filter_params = [fp for fp in filter_params if fp != ""]
 
         if filter_exp is None:
-            filter_exp = " AND ".join(paramList)
+            filter_exp = " AND ".join(filter_params)
         else:
-            filter_exp += " AND " + " AND ".join(paramList)
+            filter_exp += " AND " + " AND ".join(filter_params)
 
         params = {
             "q": q,
@@ -522,7 +576,7 @@ class MarketData:
         }
         return get_data(
             path=f"{self._ref_path}",
-            df_fn=self._search_to_def,
+            df_fn=self._search_to_df,
             paginate_fn=self._ref_paginate,
             params=params,
             paginate=paginate,
@@ -553,6 +607,5 @@ class MarketData:
         params = {"subscribed_only": subscribed_only}
 
         return get_data(
-            path="market-data/reference-data/v3/mdc",
-            params=params,
+            path="market-data/reference-data/v3/mdc", params=params, raw=raw
         )
